// Borland C++ for OS2 - (C) Copyright 1992 by Borland International
//
// Presentation Manager HELLO WORLD

#define INCL_PM                         // Define to "switch on" PM includes
                                        //   in "os2.h"
#include <os2.h>                        // PM includes
#include "pmhello.h"                    // Resource file symbolics
#include <string.h>                     // String functions

#undef NULL
#define NULL 0L

// Function prototype for WndProc callback.
// This should be the entry point for all window classes

MRESULT EXPENTRY _export WndProc(HWND hWnd,ULONG iMessage,
    MPARAM mParam1, MPARAM mParam2);


// Abstract class for all app windows
class Window
{
protected:
    HWND hWndFrame;
    HWND hWndClient;
public:
    // Provide (read) access to the windows Client or Frame
    // handles in case they're needed.
    HWND GetFrameHandle(void) { return hWndFrame; }
    HWND GetClient(void) { return hWndClient; }

    BOOL ShowWindow(int nCmdShow) { return WinShowWindow(hWndFrame,nCmdShow); }
    BOOL Update(void) { return WinUpdateWindow(hWndFrame); }

    // Pure virtual function makes "Window" an abstract class
    virtual MRESULT WndProc(HWND hWnd, ULONG iMessage, MPARAM mParam1, MPARAM mParam2 ) = 0;
};


// Class main is a container of program global functions and variables
class Main
{
public:
    static HAB      hab;
    static HMQ      hmq;
    static char *szHi;
    static char *szHello;
    static HPOINTER icon[4];
    static Window   *tempthis;          // Used to hold the "this" pointer
                                        // that _export WndProc uses to vector
                                        // to the correct member function
                                        // WndProc

    // Standard PM Initilization sequence
    static BOOL Initialize(void) {
        if ((hab = WinInitialize(NULL)) == (HAB)NULL) return FALSE;
        if ((hmq = WinCreateMsgQueue(NULL,0)) == (HMQ)NULL) return FALSE;
        return TRUE;
    }

    static BOOL MessageLoop(void);

    // Standard PM Termination sequence
    static void Terminate(void) {
        WinDestroyMsgQueue( hmq );
         WinTerminate( hab );    
    }
};

// Declare and initialize all globals in Main
HAB Main::hab;
HMQ Main::hmq;
Window *Main::tempthis = NULL;
HPOINTER Main::icon[4];
char *Main::szHi = "Hello!";
char *Main::szHello = "Borland C++ welcomes you to the wonderful world of Presentation Manager Programming!";


// Standard PM Message Loop
BOOL Main::MessageLoop(void) {
    BOOL rc;
    QMSG qmsg;
    while((rc = (BOOL)WinGetMsg(hab,&qmsg,(HWND)NULL,0,0))!=FALSE)
        WinDispatchMsg(hab,&qmsg);
    return rc;
}


class MainWindow : public Window
{
private:
    static char szClassName[14];
    static BOOL initialized;
public:
    // Unlike Windows, in PM window class registration always
    // should happen within the app before the first time a
    // window is created. "initialized" keeps track of this
    // The first class Register (and thus the first creation of a
    // Mainwindow instance) must happen AFTER Main::Initialize()
    // has been called.
    static void Register(void)
    {
        if (!WinRegisterClass(
            Main::hab,
            (PSZ)szClassName,
            ::WndProc,              // use _export WndProc
                                    // which will vector the
                                    // call to MainWindow::WndProc
            CS_SIZEREDRAW,
            sizeof(Window *)        // Allocate space for the "this"
                                    // pointer after the end of the
                                    // window record PM creates.
                                    // This is used to vector to
                                    // MainWindow::WndProc
            )) 
                AbortHello(NULL,NULL);
        initialized = TRUE;
    }
    MainWindow(void)
    {
        if (initialized == FALSE) Register();
        ULONG flCreate = FCF_STANDARD & // Set frame control flags
             ~FCF_SHELLPOSITION;

        Main::tempthis = this;          // Stuff "this" pointer in global
                                        // for use by _export WndProc
                                        // _export WndProc is called
                                        // directly as a result of
                                        // WndCreateStdWindow and will
                                        // shove this in the PM window
                                        // record and use it to vector to
                                        // MainWindow::WndProc

        if ((hWndFrame = WinCreateStdWindow(
               HWND_DESKTOP,            // Desktop window is parent
               0,
               &flCreate, 
               (PSZ)szClassName,
               (PSZ)"",
               0, 
               (HMODULE)NULL,           // Resource is in .EXE file
               ID_WINDOW,               // Frame window identifier 
               &hWndClient 
               )) == (HWND)NULL) AbortHello(hWndFrame, hWndClient);

        // Set title bar fo window
        WinSetWindowText(hWndFrame, (PSZ)"Hello, World!");

        if (!WinSetWindowPos( hWndFrame,
                       HWND_TOP,            // window at position 20, 100,
                       20, 100, 600, 300,   // and size 600, 300.
                       // Flags set to show and activate frame window
                       SWP_SIZE | SWP_MOVE | SWP_ACTIVATE | SWP_SHOW
                     ))
        AbortHello(hWndFrame, hWndClient); /* Terminate the application    */
    }
    // Destory window when object goes out of scope
    MRESULT WndProc(HWND hWnd,ULONG iMessage,MPARAM mParam1,MPARAM mParam2);
    ~MainWindow(void){
        WinDestroyWindow(hWndFrame);
    }
};

char MainWindow::szClassName[14] = "MyWindow";  // Must be unique in the app
BOOL MainWindow::initialized = FALSE;

// MainWindow object window procedure
MRESULT MainWindow::WndProc(HWND hWnd,ULONG iMessage,MPARAM mParam1,
    MPARAM mParam2 )
{
    switch( iMessage )
    {
      case WM_CREATE:
        // Do window initialization stuff here
        // WinLoadPointer loads icons from the exe. 
        Main::icon[1] = WinLoadPointer(HWND_DESKTOP,(HMODULE)NULL,ID_ICON1);
        Main::icon[2] = WinLoadPointer(HWND_DESKTOP,(HMODULE)NULL,ID_ICON2);
        Main::icon[3] = WinLoadPointer(HWND_DESKTOP,(HMODULE)NULL,ID_ICON3);
        break;

      case WM_COMMAND:
        // Menu selections end up here
        // On menu selections 1,2,or 3 set Main::icon[0] to the
        // proper icon, then call WinInvalidateRegion to force
        // PM to generate a WM_PAINT message
        {
        USHORT command; 
        command = SHORT1FROMMP(mParam1);      // Extract the command value
        switch (command)
        {
          case ID_OPTION1:
            Main::icon[0] = Main::icon[1];
            WinInvalidateRegion( hWnd, (HRGN)NULL, FALSE );
            break;
          case ID_OPTION2:
            Main::icon[0] = Main::icon[2];
            WinInvalidateRegion( hWnd, (HRGN)NULL, FALSE );
            break;
          case ID_OPTION3:
            Main::icon[0] = Main::icon[3];
            WinInvalidateRegion( hWnd, (HRGN)NULL, FALSE );
            break;
          case ID_EXITPROG:
            WinPostMsg( hWnd, WM_CLOSE, 0, 0 );
            break;
          default:
            return WinDefWindowProc( hWnd, iMessage , mParam1, mParam2 );
        }
    
        break;
        }
      case WM_ERASEBACKGROUND:
        // Return TRUE to cause PM to paint the window background
        // in SYSCLR_WINDOW.
        return (MRESULT)( TRUE );
      case WM_PAINT:
        // Window draw stuff here
        {
          HPS    hps;
          RECTL  rc;
          POINTL pt;

          hps = WinBeginPaint( hWnd, (HPS)NULL, &rc );
          GpiSetColor( hps, CLR_NEUTRAL );         // set text color
          GpiSetBackColor( hps, CLR_BACKGROUND );  // set text background
          GpiSetBackMix( hps, BM_OVERPAINT );      // set text mix
          WinQueryWindowRect(hWnd,&rc);            // get window bounds

          // Draw hello message repeatedly from near the top 
          // of the window to the middle
          pt.x = 6;
          for (int i=rc.yTop-10;i>rc.yTop/2;i--){
            pt.y = i;
            GpiCharStringAt( hps, &pt, strlen(Main::szHello),
                             (PSZ)Main::szHello);
          }

          // If icons are loaded, display the proper one
          // otherwise display the Hi message.
          if (Main::icon[0] != (HMODULE) NULL) {
            WinDrawPointer(hps,rc.xRight/2-16,rc.yTop/2+16, 
                Main::icon[0],DP_NORMAL);
          } else {
            pt.x = rc.xRight/2-16; pt.y = rc.yTop/2+16;        
            GpiCharStringAt( hps, &pt, strlen(Main::szHi),
                             (PSZ)Main::szHi);
          }
    
          WinEndPaint( hps );  
          break;
        }
      case WM_CLOSE:
        // Do termination stuff and cleanup here
        WinPostMsg( hWnd, WM_QUIT, 0, 0 ); // Cause termination
        break;
      default:
          return WinDefWindowProc( hWnd, iMessage , mParam1, mParam2 );
      }
      return FALSE;
}

// main entry point
void cdecl main()
{
    // Initialize PM first
    if (Main::Initialize() == FALSE)
        AbortHello((HWND)NULL,(HWND)NULL);

    {
        MainWindow MainWnd; // Set up new scope and create an instance
                            // of MainWindow, which causes it to
                            // Register the class and Create the
                            // main window.  When MainWnd goes out
                            // of scope, the destructor will be called
                            // which cleans up with WinDestroyWindow
        Main::MessageLoop();
    }
    Main::Terminate();
}


MRESULT EXPENTRY _export WndProc(HWND hWnd,ULONG iMessage,
    MPARAM mParam1, MPARAM mParam2)
{
    // Pointer to the c++ window object
    Window *pWindow = (Window *)WinQueryWindowULong(hWnd,0);

    // The pointer pWindow will have an invalid value if the WM_CREATE
    // message has not yet been processed (we respond to the WM_CREATE
    // message by setting the extra bytes to be a pointer to the
    // (C++) object corresponding to the Window identified
    // by hWnd).  The messages that
    // precede WM_CREATE must be processed without using pWindow so we
    // pass them to WinDefWindowProc.

    if (pWindow == 0) {
        if (iMessage == WM_CREATE) {
            pWindow = Main::tempthis;
            WinSetWindowULong(hWnd,0,(ULONG)pWindow);
            return pWindow->WndProc(hWnd,iMessage,mParam1,mParam2);
        } else
            return WinDefWindowProc(hWnd,iMessage,mParam1,mParam2);
    } else
        return pWindow->WndProc(hWnd,iMessage,mParam1,mParam2);
}

// AbortHello -- report an error returned from an API service.
VOID AbortHello(HWND hwndFrame,HWND hwndClient)
{
PERRINFO  pErrInfoBlk;
PSZ       pszOffSet;
void      stdprint(void);
PSZ  pszErrMsg;

      DosBeep(100,10);
      if ((pErrInfoBlk = WinGetErrorInfo(Main::hab)) != NULL)
      {
        pszOffSet = ((PSZ)pErrInfoBlk) + pErrInfoBlk->offaoffszMsg;
        pszErrMsg = ((PSZ)pErrInfoBlk) + *((PSHORT)pszOffSet);
        if (hwndFrame && hwndClient)
            WinMessageBox(HWND_DESKTOP,      // Parent window is desk top
                       hwndFrame,            // Owner window is our frame
                       (PSZ)pszErrMsg,       // PMWIN Error message
                       (PSZ)"Error Msg",     // Title bar message
                       MSGBOXID,             // Message identifier
                       MB_MOVEABLE | MB_CUACRITICAL | MB_CANCEL ); // Flags
        else
            WinMessageBox(HWND_DESKTOP,      // Parent window is desk top
                       HWND_DESKTOP,         // Owner window is desktop
                       (PSZ)pszErrMsg,       // PMWIN Error message
                       (PSZ)"Error Msg",     // Title bar message
                       MSGBOXID,             // Message identifier
                       MB_MOVEABLE | MB_CUACRITICAL | MB_CANCEL ); // Flags
        WinFreeErrorInfo(pErrInfoBlk);
      }
      WinPostMsg(hwndClient, WM_QUIT, (ULONG)0, (ULONG)0);
}
